Fourier = {
    x: 0, y: 0, lx: 0, ly: 0, px: 0, py: 0,
    circles: [],
    lines: [],
    gui: new Gui(),
    createCircle: function(shape, size) {
        var theta = 2 * Math.PI / 50
        var cos = Math.cos(theta)
        var sin = Math.sin(theta)

        var z
        var x = 1
        var y = 0

        for (var i = 0; i < 50 + 1; i++) {
            shape.addVertex(x * size, y * size)
            z = x
            x = cos * x - sin * y
            y = sin * z + cos * y
        }
        shape.addVertex(0, 0)
    },

    points: [
        [0, -10],
        [0, -20],
        [0, -30],
        [-10, -40],
        [-20, -50],
        [-30, -50],
        [-40, -50],
        [-50, -40],
        [-60, -30],
        [-60, -20],
        [-60, -10],
        [-60, 0],
        [-60, 10],
        [-60, 20],
        [-60, 30],
        [-50, 40],
        [-40, 50],
        [-30, 50],
        [-20, 50],
        [-10, 50],
        [0, 50],
        [10, 50],
        [20, 50],
        [30, 50],

        [40, 50],
        [40, 40],
        [40, 30],
        [40, 20],
        [40, 10],
        [40, 0],
        [40, -10],
        [40, -20],
        [40, -30],
        [50, -30],
        [60, -30],
        [60, -40],
        [60, -50],
        [50, -50],
        [40, -50],
        [30, -50],
        [20, -50],
        [10, -50],
        [0, -50],
        [0, -40],
        [0, -30],
        [10, -30],
        [20, -30],
        [20, -20],
        [20, -10],
        [20, 0],
        [20, 10],
        [20, 20],
        [20, 30],
        [10, 30],
        [0, 30],
        [0, 20],
        [0, 10],
        [-10, 10],
        
        [-20, 10],
        [-20, 20],
        [-20, 30],
        [-30, 40],
        [-40, 30],
        [-40, 20],
        [-40, 10],
        [-40, 0],
        [-40, -10],
        [-40, -20],
        [-40, -30],
        [-30, -40],
        [-20, -30],
        [-20, -20],
        [-20, -10],
        [-10, -10]
    ],
    epicycles: 34
}

register("command", function() {

    for (var n = -Fourier.epicycles; n <= Fourier.epicycles; n++) {
        var coeff = [0, 0]

        for (var i = 0; i < Fourier.points.length; i++) {
            var tx = Fourier.points[i][0]
            var ty = Fourier.points[i][1]

            var tang = i / Fourier.points.length * 2 * Math.PI
            var nt = tang * n

            var x = tx * Math.cos(nt) + ty * Math.sin(nt)
            var y = ty * Math.cos(nt) - tx * Math.sin(nt)

            x /= Fourier.points.length
            y /= Fourier.points.length

            coeff[0] += x
            coeff[1] += y
        }
        
        var theta = Math.atan2(coeff[1], coeff[0]) * (180 / Math.PI)
        var r = Math.sqrt(coeff[0] * coeff[0] + coeff[1] * coeff[1])
        print("n: " + n + "    t: " + theta + "   r: " + r + "   x: " + coeff[0] + "   y: " + coeff[1])
        Fourier.circles.push(new FCircle(r, n, theta))
    }

    Fourier.circles.sort(function(a, b) {return b.size - a.size})

    Fourier.gui.open()
}).setName("fourier")

Fourier.gui.registerDraw(function() {
    for (var i = 0; i < Fourier.circles.length; i++) {
        Fourier.circles[i].draw()
    }
    
    for (var i = 0; i < Fourier.points.length; i++) {
        Renderer.translate(Renderer.screen.getWidth() / 2, Renderer.screen.getHeight() / 2)
        Renderer.translate(Fourier.points[i][0], Fourier.points[i][1])
        Renderer.drawRect(0xff00ff00, -1, -1, 2, 2)
    }

    for (var i = 0; i < Fourier.lines.length; i++) {
        Fourier.lines[i].draw()
    }

    Renderer.translate(Fourier.x, Fourier.y)
    Renderer.drawRect(0xffff0000, -1, -1, 2, 2)

})

register("step", function() {
    if (!Fourier.gui.isOpen()) return

    Fourier.x = Renderer.screen.getWidth() / 2
    Fourier.y = Renderer.screen.getHeight() / 2
    for (var i = 0; i < Fourier.circles.length; i++) {
        Fourier.circles[i].step()
        Fourier.px = Fourier.x
        Fourier.py = Fourier.y

        Fourier.x = Fourier.px + Fourier.circles[i].size * Math.cos(Fourier.circles[i].angle * Math.PI / 180)
        Fourier.y = Fourier.py + Fourier.circles[i].size * Math.sin(Fourier.circles[i].angle * Math.PI / 180)
        Fourier.circles[i].x = Fourier.px
        Fourier.circles[i].y = Fourier.py
    }

    if (Fourier.lx != 0 && Fourier.ly != 0) {
        Fourier.lines.push(new FLine(Fourier.lx, Fourier.ly, Fourier.x, Fourier.y))
    }
    while (Fourier.lines.length > 510) {Fourier.lines.shift()}
    for (var i = 0; i < Fourier.lines.length; i++) {
        Fourier.lines[i].step()
    }

    Fourier.lx = Fourier.x
    Fourier.ly = Fourier.y
})

FCircle = function(size, speed, angle) {
    this.size = size
    this.speed = speed
    this.angle = angle

    this.x = Renderer.screen.getWidth() / 2
    this.y = Renderer.screen.getHeight() / 2

    this.shape = new Shape(0xff0000ff)
    Fourier.createCircle(this.shape, this.size)
    this.shape.setDrawMode(2)

    this.step = function() {
        this.angle += speed
        while(this.angle > 360) this.angle -= 360
    }

    this.draw = function() {
        Renderer.translate(this.x, this.y)
        Renderer.rotate(this.angle)
        this.shape.draw()
    }
}

FLine = function(x1, y1, x2, y2) {
    this.alpha = 255

    this.line = new Shape(0xffffffff)
    this.line.addVertex(x1, y1)
    this.line.addVertex(x2, y2)
    this.line.setDrawMode(1)

    this.step = function() {
        this.alpha -= 0.5
        this.line.setColor(Renderer.color(255, 255, 255, this.alpha))
    }

    this.draw = function() {
        this.line.draw()
    }
}